package com.example.chessboardtest;

import androidx.appcompat.app.AppCompatActivity;

import com.chessnutech.chessnutevo.ChessnutServiceChessBoardCallback;
import com.chessnutech.chessnutevo.ChessnutServiceChessnutVisionCallback;
import com.chessnutech.chessnutevo.ChessnutServiceInterface;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.IBinder;
import android.os.RemoteException;
import android.provider.Settings;
import android.util.Log;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.RandomAccessFile;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

public class MainActivity extends AppCompatActivity implements ServiceConnection {
    private static final String TAG = "MainActivity";
    private ChessnutServiceInterface chessnutServiceInterface;
    private String chessnutVisionCallbackUUID = "";
    private boolean[] switches = new boolean[64];
    private byte[] rArray = new byte[64];
    private byte[] gArray = new byte[64];
    private byte[] bArray = new byte[64];
    private int ledStep;
    private int rgbStep;

    private final ChessnutServiceChessBoardCallback chessnutServiceChessBoardCallback = new ChessnutServiceChessBoardCallback.Stub() {
        @Override
        public void onFenChanged(String fen) {
            //this callback triggered when fen change on board
            Log.d(TAG, "onFenChanged: " + fen);
        }
    };

    private final ChessnutServiceChessnutVisionCallback chessnutServiceChessnutVisionCallback = new ChessnutServiceChessnutVisionCallback.Stub() {
        @Override
        public void onDetectionResult(boolean valid, float[] x, float[] y, float[] w, boolean promote, String fen, int[] stars) throws RemoteException {
            //this callback triggered when the detection is ended
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        //bind service
        Intent si = new Intent();
        si.setComponent(new ComponentName("com.chessnutech.chessnutevo.launcher", "com.chessnutech.chessnutevo.chessnutservice.ChessnutService"));
        bindService(si, this, Context.BIND_AUTO_CREATE);

        //led task
        ScheduledExecutorService timerExecutor = Executors.newSingleThreadScheduledExecutor();
        timerExecutor.scheduleAtFixedRate(new Runnable() {
            @Override
            public void run() {
                if (chessnutServiceInterface != null) {
                    try {
                        //read fen
//                        if (!chessnutServiceInterface.getFen().equals("rnbqkbnr/pppppppp/pppppppp/rnbqkbnr/RNBQKBNR/PPPPPPPP/PPPPPPPP/RNBQKBNR")) {
//                            //error!!!
//                            Log.d(TAG, "fen not match");
//                        }
                        //set led
                        for (int i=0;i<8;i++) {
                            for (int j=0;j<8;j++) {
                                switches[i*8+j] = true;
                                rArray[i*8+j] = 0;
                                gArray[i*8+j] = 0;
                                bArray[i*8+j] = 0;
                                if (rgbStep == 0) {
                                    if ((i+j) / 2 <= ledStep) rArray[i*8+j] = (byte) (0xff >> (ledStep*2-i-j));
                                } else if (rgbStep == 1) {
                                    if ((i+j) / 2 <= ledStep) gArray[i*8+j] = (byte) (0xff >> (ledStep*2-i-j));
                                } else if (rgbStep == 2) {
                                    if ((i+j) / 2 <= ledStep) bArray[i*8+j] = (byte) (0xff >> (ledStep*2-i-j));
                                } else {
                                    if (ledStep < 2) {
                                        rArray[i * 8 + j] = ((i+j) % 2 == 0) ? (byte) 0xff : 0;
                                        gArray[i * 8 + j] = ((i+j) % 2 == 0) ? (byte) 0xff : 0;
                                        bArray[i * 8 + j] = ((i+j) % 2 == 0) ? (byte) 0xff : 0;
                                    } else if (ledStep < 4) {
                                        rArray[i * 8 + j] = ((i+j) % 2 == 1) ? (byte) 0xff : 0;
                                        gArray[i * 8 + j] = ((i+j) % 2 == 1) ? (byte) 0xff : 0;
                                        bArray[i * 8 + j] = ((i+j) % 2 == 1) ? (byte) 0xff : 0;
                                    } else if (ledStep < 6) {
                                        rArray[i * 8 + j] = (byte) 0xff;
                                        gArray[i * 8 + j] = (byte) 0xff;
                                        bArray[i * 8 + j] = (byte) 0xff;
                                    }
                                }
                            }
                        }
                        //set led command
                        //if the parameter is null, means no changes.
                        chessnutServiceInterface.setLed(switches, rArray, gArray, bArray);
                        ledStep++;
                        if (ledStep > 7) {
                            ledStep = 0;
                            rgbStep++;
                            if (rgbStep > 3) rgbStep = 0;
                        }
                        //read fen to compare
                    } catch (RemoteException e) {
                        throw new RuntimeException(e);
                    }
                }
            }
        }, 0, 300, TimeUnit.MILLISECONDS);
    }

    @Override
    public void onServiceConnected(ComponentName name, IBinder service) {
        chessnutServiceInterface = ChessnutServiceInterface.Stub.asInterface(service);
        //chess board related
        try {
            //register fen change callback
            chessnutServiceInterface.registerChessBoardCallback(chessnutServiceChessBoardCallback);
            //get fen manually
            String fen = chessnutServiceInterface.getFen();
            Log.d(TAG, "onServiceConnected: fen = " + fen);
        } catch (RemoteException e) {
            throw new RuntimeException(e);
        }
        //chessnut vision related
        try {
            //please mark down your uuid, you will need this to do detection
            chessnutVisionCallbackUUID = chessnutServiceInterface.registerChessnutVisionCallback(chessnutServiceChessnutVisionCallback);
            chessnutServiceInterface.registerChessBoardCallback(chessnutServiceChessBoardCallback);
            //use the following code for chess vision detection
            //chessnutServiceInterface.chessDetect(bitmap, chessnutVisionCallbackUUID);
        } catch (RemoteException e) {
            Log.e(TAG, Log.getStackTraceString(e));
        }
    }

    @Override
    public void onServiceDisconnected(ComponentName name) {

    }
}